#!/usr/bin/env python

"""
Construct free/busy records for a user, either recording that user's own
availability schedule or the schedule of another user (using details provided
when scheduling events with that user).

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.data import get_window_end, Object
from imiptools.dates import get_default_timezone
from imiptools.profile import Preferences
from imip_store import FileStore, FilePublisher
import sys

# Main program.

if __name__ == "__main__":

    # Interpret the command line arguments.

    try:
        user = sys.argv[1]
        args = sys.argv[2:]
        participant = args and args[0] not in ("-n", "-s", "-v") and args[0] or user
        store_and_publish = "-s" in args
        include_needs_action = "-n" in args
        verbose = "-v" in args
    except IndexError:
        print >>sys.stderr, """\
    Need a user and an optional participant (if different from the user),
    along with the -s option if updating the store and the published details.
    """
        sys.exit(1)

    preferences = Preferences(user)
    tzid = preferences.get("TZID") or get_default_timezone()

    # Get the size of the free/busy window.

    try:
        window_size = int(preferences.get("window_size"))
    except (TypeError, ValueError):
        window_size = 100
    window_end = get_window_end(tzid, window_size)

    store = FileStore()
    publisher = FilePublisher()

    # Get identifiers for uncancelled events.

    all_events = store.get_active_events(user)

    # Obtain event objects.

    objs = []
    for uid, recurrenceid in all_events:
        if verbose:
            print >>sys.stderr, uid, recurrenceid
        event = store.get_event(user, uid, recurrenceid)
        if event:
            objs.append(Object(event))

    # Build a free/busy collection for the given user.

    fb = []
    for obj in objs:
        partstat = obj.get_participation_status(participant)
        recurrenceids = not obj.get_recurrenceid() and store.get_recurrences(user, obj.get_uid())

        if obj.get_participation(partstat, include_needs_action):
            for p in obj.get_active_periods(recurrenceids, tzid, window_end):
                fb.append(obj.get_freebusy_period(p, partstat == "ORG"))

    fb.sort()

    # Store and publish the free/busy collection.

    if store_and_publish:
        if user == participant:
            store.set_freebusy(user, fb)
            publisher.set_freebusy(user, fb)
        else:
            store.set_freebusy_for_other(user, fb, participant)
    else:
        for item in fb:
            print "\t".join(item.as_tuple(strings_only=True))

# vim: tabstop=4 expandtab shiftwidth=4
